# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
from novalinkAutoInstaller.src.EndPoint import HMC_Handler
from novalinkAutoInstaller.src.Validate import validateVios
from novalinkAutoInstaller.src.Validate import validateNovalink
from novalinkAutoInstaller.src.Install import installVios
from novalinkAutoInstaller.src.Install import installNovalink
from multiprocessing import Process
import subprocess
import os
import sys
import time
import six
from six.moves import configparser
import openpyxl
CONF = configparser.ConfigParser()
CONF.read(autoLogger.configPath)
log = autoLogger.AutoLogger("NovalinkAutoInstallModule")


# Validating duplicate IP assignment to VIOS IP Parameters
# Validate IP Clash using Ping Test
# Validating mismatch subnet mask for all VIOS
# Validating mismatch gateway IP for all VIOS
def validateIPAddress(viosValidationList, NLinfo):
    log.info("Inside validateIPAddress function")
    IP_ERROR = False
    IPlist = []
    IPlist.append(NLinfo['GATEWAY'])
    IPlist.append(NLinfo['SUBNET MASK'])
    IPlist.append(NLinfo['BOOTP SERVER IP'])
    NLIpAdd = NLinfo['IP ADDRESS']
    if NLIpAdd in IPlist:
        print(MSGCATALOG.SAME_IP_ASSIGN_ERROR.format(NLIpAdd))
        IP_ERROR = True
    else:
        IPlist.append(NLIpAdd)
    for viosInfo in viosValidationList:
        viosIPAdd = viosInfo['IP ADDRESS']
        IPlist.append(viosInfo['GATEWAY'])
        IPlist.append(viosInfo['SUBNET MASK'])
        if viosIPAdd in IPlist:
            print(MSGCATALOG.SAME_IP_ASSIGN_ERROR.format(viosIPAdd))
            IP_ERROR = True
            continue
        else:
            IPlist.append(viosIPAdd)
    return IP_ERROR


# Checking existing VM on Managed system
# If exist, print VM names and exit installation
def checkExistingLPAR(viosValidationList, installvios):
    log.info("Inside checkExistingLPAR function")
    for viosInfo in viosValidationList:
        isLparExist = installvios.checkExistingLpar(viosInfo['HOST NAME'])
        return isLparExist


# VIOS partition and profile creation
# Ping test check for VIOS IPs
# In case of Ping failed, deleting VIOS profile
def profAndNetworkVald(
        viosValidationList, installvios, viosInstallerList, HMC_IP):
    log.info("Inside profAndNetworkVald function")
    viosId = int(CONF.get(
        'vios_profile', 'VIOS_ID_START_INDEX'))
    viosSlotNum = int(CONF.get(
        'vios_profile', 'VIOS_SLOT_NUM_START_INDEX'))
    trunkPriority = int(CONF.get(
        'vios_profile', 'VIOS_TRUNK_PRIORITY_START_INDEX'))
    lparSlotNum = int(CONF.get(
        'novalink_profile', 'LPAR_SLOT_NUM_START_INDEX'))
    novalinkLparId = CONF.get('novalink_profile', 'LPAR_ID')
    deleteVIOS = False
    print("\nCreating VIOS Profile, Please wait...")
    for viosInfo in viosValidationList:
        viosInfo['hmcIp'] = HMC_IP
        viosInfo['viosId'] = viosId
        viosInfo['serverIp'] = HMC_IP
        viosInfo['viosSlotNum'] = viosSlotNum
        viosInfo['lparSlotNum'] = lparSlotNum
        viosInfo['novalinkLparId'] = novalinkLparId
        viosInfo['trunkPriority'] = trunkPriority
        installerInfo = installvios.viosValidation(viosInfo)
        if not installerInfo:
            return False
        viosInstallerList.append(installerInfo)
        if not installerInfo['profileCreationStatus']:
            print("Error: Ubanle to create VIOS profile.")
            return False
        if not installerInfo['pingTest']:
            print("Ping failed, trying with default adapter")
        viosId += 1
        viosSlotNum += 1
        lparSlotNum += 1
        trunkPriority += 1
    return True


# VIOS installation
def installVIOS(viosInstallerList, installvios):
    log.info("Inside installVIOS function")
    for viosInfo in viosInstallerList:
        res = installvios.installVIOS(viosInfo)
        if res['code'] == 0:
            viosInfo['installationTriggered'] = True
        else:
            print("Error: VIOS Installation Failed")
            log.error("VIOS Installation failed")
            return False
    return True


# Checking RMC Status every 15 seconds
# Default setup for timeout is 10 min in config file
# So 10 min*4 = 40 times loops and waiting time is 15 seconds
# So 4 loop is equal to 1 minutes, means 40 loops = 10 min
# getting active, it should exit installation
def checkRMC(viosInstallerList, installvios):
    log.info("Inside checkRMC function")
    print(MSGCATALOG.WAITING_FOR_RMC_ACTIVE)
    msg = "Please wait"
    timeOut = int(CONF.get('vios_profile', 'VIOS_RMC_UP_TIMEOUT'))*4
    for x in range(timeOut):
        allVIOSRMCActive = True
        for viosInfo in viosInstallerList:
            if viosInfo['installationTriggered']:
                if not viosInfo['isRmcActive']:
                    allVIOSRMCActive = False
                    frameName = viosInfo['frameName']
                    viosName = viosInfo['viosName']
                    res = installvios.getRmcState(frameName, viosName)
                    msg = msg+"..."
                    sys.stdout.write("%s   \r" % (msg))
                    sys.stdout.flush()
                    log.info("RMC State for "+viosName+" :"+res['rmc_state'])
                    if res['rmc_state'] == "active":
                        print("\nInfo: RMC state for "+viosName+" is : "+str(
                            res['rmc_state']))
                        viosInfo['isRmcActive'] = True
                        isConfigured = configVIOS(viosInfo, installvios)
                        if isConfigured:
                            log.info("VIOS Configured successfully")
                        else:
                            print(MSGCATALOG.VIOS_CONFIGURE_ERROR.format(
                                viosName))
                            log.error("Unable to Configure VIOS "+viosName)
                            return False

        time.sleep(15)
        if allVIOSRMCActive:
            log.info("RMC active for all VIOS")
            return allVIOSRMCActive
    print(MSGCATALOG.VIOS_RMC_INACTIVE_TIMEOUT_ERROR)
    log.error("Request timeout, RMC is not active")
    return False


# Configure VIOS
# Set New Password
# Deploy Rules
# Restart VIOS
def configVIOS(viosInfo, installvios):
    frameName = viosInfo['frameName']
    viosName = viosInfo['viosName']
    login = CONF.get('vios_profile', 'VIOS_DEFAULT_LOGIN')
    password = CONF.get('vios_profile', 'VIOS_NEW_PASSWORD')
    print(MSGCATALOG.VIOS_CONFIGURE_MSG.format(viosName))
    allVIOSConfigured = False
    isConfigured = installvios.configureVIOS(
        frameName, viosName, login, password)
    return isConfigured


# Ping Test after VIOS Restart
# Default setup for timeout is 8 min in config file
# So 8 min*4 = 32 times loops and waiting time is 15 seconds
# So 4 loop is equal to 1 minutes, means 32 loops = 8 min
# pingable, it should exit installation
def restartAndCreateSEA(viosInstallerList, installvios):
    log.info("Inside restartStatusCheck function")
    SEACreatedCount = 0
    print("Checking VIOS status")
    time.sleep(10)
    msg = "Please wait"
    timeOut = int(CONF.get('vios_profile', 'VIOS_RESTART_PING_TEST_TIMEOUT'))*4
    for x in range(timeOut):
        for viosInfo in viosInstallerList:
            ipAdd = viosInfo['viosIp']
            viosName = viosInfo['viosName']
            res = installvios.pingTest(ipAdd)
            msg = msg+"..."
            sys.stdout.write("%s   \r" % (msg))
            sys.stdout.flush()
            log.info("Ping test ResCode for "+viosName+" :"+str(res['code']))
            if res['code'] == 0:
                if not viosInfo['isSEAcreated']:
                    print("Info: "+viosName+" Restarted")
                    time.sleep(30)
                    isSEACreated = createSEA(viosInfo, installvios)
                    if isSEACreated:
                        viosInfo['isSEAcreated'] = True
                        SEACreatedCount += 1
                        print(MSGCATALOG.VIOS_CREATE_SEA_SUCCESS.format(
                            viosName))
                        log.info("SEA Created successfully")
                    else:
                        print(MSGCATALOG.VIOS_CREATE_SEA_ERROR.format(
                            viosName))
                        log.error("Unable to create SEA")
                        return False
        if SEACreatedCount == len(viosInstallerList):
            return True
        else:
            time.sleep(15)
    print(MSGCATALOG.VIOS_RESTART_TIMEOUT)
    log.error("Request timeout, Unable to restart VIOS")
    return False


# Create SEA
# Get virtual adapters
# Get PORT VLAN ID
# Get Physical Adapter
# Create SEA using virtual, physical adapters and PORT VLAN ID
def createSEA(viosInfo, installvios):
    log.info("Inside createSEA function")
    frameName = viosInfo['frameName']
    viosName = viosInfo['viosName']
    vLanID = viosInfo['vlanId']
    print(MSGCATALOG.VIOS_CREATE_SEA_MSG.format(viosName))
    isSEACreated = installvios.configureSEA(
        frameName, viosName, vLanID)
    return isSEACreated


def add_private_network(viosInstallerList, installvios):
    log.info("Inside add_private_network function")
    add_network_count = 0
    time.sleep(2)
    ip_add = CONF.get(
            'vios_profile', 'VIOS_PRIVATE_IP_START')
    ip_count = 0
    for viosInfo in viosInstallerList:
        try:
            ip_add_split = ip_add[0:12]
            last_ip_digit = ip_add[-1]
            next_last_ip = int(last_ip_digit)+ip_count
            ip_add = ip_add_split+str(next_last_ip)
            ip_count = ip_count+1
            is_network_created = installvios.add_network_vios(viosInfo, ip_add)
            if is_network_created:
                log.info("Private network added on :"+str(
                    viosInfo['viosName']))
                print(MSGCATALOG.PRIVATE_NETWORK_ADDED_SUCCESS.format(
                    viosInfo['viosName']))
            else:
                log.error("Unable to create private network on :"+str(
                    viosInfo['viosName']))
                print(MSGCATALOG.PRIVATE_NETWORK_ADDING_FAILED.format(
                    viosInfo['viosName']))
        except Exception as e:
            log.error("Unable to add network, error as : "+str(e))
            print(MSGCATALOG.PRIVATE_NETWORK_ADDING_FAILED.format(
                viosInfo['viosName']))


def configureSRIOV(viosInstallerList, installvios):
    isAllSRIOVAssigned = True
    for info in viosInstallerList:
        res = installvios.configureSRIOV(info)
        if not res:
            isAllSRIOVAssigned = False
    return isAllSRIOVAssigned


# Validate VIOS ISO path
def validateISOPath(hmcObj, path):
    log.info("Inside validateISOPath function")
    validateObj = validateVios.VIOSInputValidator(hmcObj)
    res = validateObj.valdISOPath(path)
    return res


# Validation of VIOS information
# In case of error, printing error message and exit
def validateViosInstallerInfo(hmcObj, info, viosHeaderName):
    log.info("Inside validateViosInstallerInfo function")
    validateObj = validateVios.VIOSInputValidator(hmcObj)
    res = validateObj.validateViosInfo(info, viosHeaderName)
    return res


# Parsing IO Adapters list
def parseIOadapters(info):
    log.info("Inside parseIOadapters function")
    iolist = []
    for dataKey, dataVal in info.items():
        if '|' in dataKey:
            iolist.append({dataKey: dataVal})
    return iolist


# Validation of IO Adapters
def validateIOinfo(hmcObj, iolist, viosInstallerInfo, vios_count):
    log.info("Inside validateIOinfo function")
    viosName = viosInstallerInfo['NAME']
    validateObj = validateVios.VIOSInputValidator(hmcObj)
    res = validateObj.validateIO(iolist, viosName, vios_count)
    viosInstallerInfo['only_sriov'] = validateObj.only_sriov
    return res


def validateAdapterSelection(hmcObj, viosValidationList):
    log.info("Inside validateAdapterSelection function")
    validateObj = validateVios.VIOSInputValidator(hmcObj)
    res = validateObj.validateIOSelection(viosValidationList)
    return res


def checkPvmctlService(hmcObj):
    log.info("Inside checkPvmctlService function")
    res = hmcObj.get_pvmctl_version()
    version = ""
    if res['code'] == 0:
        for line in res['stdout']:
            version = line
            log.info("pvmctl service is up and NL version is : "+version)
            break
    if version == "":
        log.info("pvmctl service is not up : "+version)
        return False
    else:
        return True


def configureNovalink(hmcObj, hostname, NL_IP, SSH_PORT):
    log.info("Inside configureNovalink function")
    for x in range(15):
        log.info("Checking pvmctl server attempt : "+str(x))
        res = checkPvmctlService(hmcObj)
        if res:
            break
        time.sleep(10)
    code, os_name = hmcObj.configureNovalinkVm(hostname)
    if code == 0:
        UBUNTU_OS_NAME = CONF.get('novalink_profile', 'UBUNTU_OS_NAME')
        RHEL_OS_NAME = CONF.get('novalink_profile', 'RHEL_OS_NAME')
        if UBUNTU_OS_NAME in str(os_name):
            print(MSGCATALOG.NOVALINK_CONFIGURED_SUCCESS)
            print(MSGCATALOG.PRESS_ENTER_FOR_LOGIN)
        elif RHEL_OS_NAME in str(os_name):
            rhel_uname = CONF.get('novalink_profile', 'RHEL_UNAME')
            rhel_pwd = CONF.get('novalink_profile', 'RHEL_PWD')
            connection = hmcObj.createSSHconnection(
                NL_IP, SSH_PORT, rhel_uname, rhel_pwd)
            try:
                if connection.get("code") != 200:
                    log.error("Unble to create SSH Conn to RHEL Novalink VM")
            except Exception as e:
                log.info("SSH Conn to RHEL Novalink SUCCESSFUL")
                code = hmcObj.disable_firewall(hostname)
                if code == 0:
                    print(MSGCATALOG.NOVALINK_CONFIGURED_SUCCESS)
                    print(MSGCATALOG.PRESS_ENTER_FOR_LOGIN)
                else:
                    log.error("Unable to disable firewall")
    return True


def checkSshToNl(hmcObj, NL_IP, hostname, nlInstallerObj, nlInfo, hmc_cred):
    log.info("Inside checkSshToNl function")
    SSH_PORT = CONF.get('ssh_config', 'SSH_PORT')
    NL_SSH_CONN_TIMEOUT = CONF.get('novalink_profile', 'NL_SSH_CONN_TIMEOUT')
    nl_uname = CONF.get('novalink_profile', 'NL_UNAME')
    nl_pwd = CONF.get('novalink_profile', 'NL_PWD')
    if hostname == "":
        hostname = NL_IP.replace(".", "-")
    log.info("Starting loop to check NL SSH Connection")
    for x in range(int(NL_SSH_CONN_TIMEOUT)):
        log.info("Trying SSH Conn to Novalink : Attempt "+str(x))
        connection = hmcObj.createSSHconnection(
            NL_IP, SSH_PORT, nl_uname, nl_pwd)
        try:
            if connection.get("code") != 200:
                log.error("SSH to Novalink VM is not ready, trying again. \
It will try upto 300 attempts.")
        except Exception as e:
            log.info("SSH Conn to Novalink SUCCESSFUL")
            res = configureNovalink(hmcObj, hostname, NL_IP, SSH_PORT)
            return res

        if x > 1:
            time.sleep(20)
        else:
            time.sleep(300)

    log.info("Request timeout, Unable to create SSH conn to NL")
    return False


def hmcCloseConnAndExitInstall(hmcObj):
    hmcObj.closeSSHconnection()
    log.info("HMC Connection closed, exiting installation")
    exit(1)


# Main Function
def main(HMC_IP, HMC_USERNAME, HMC_PASSWORD, mode):
    log.info("Inside Novalink Auto Install main function")
    hmcObj = HMC_Handler.HMChandler()
    SSH_PORT = CONF.get('ssh_config', 'SSH_PORT')
    connection = hmcObj.createSSHconnection(
        HMC_IP, SSH_PORT, HMC_USERNAME, HMC_PASSWORD)
    # printing console message in case of SSH Authentication error
    try:
        if connection.get("code") == 201:
            print(connection['error'])
            log.error("SSH connection to HMC Failed : \
                "+str(connection['error']))
            exit(1)
    except Exception as e:
        log.error("SSH connection to HMC Failed")

    if mode == "normal":
        loc = CONF.get('install_template', 'TEMPLATE_FILE_NAME')+'.xlsx'
        try:
            wb = openpyxl.load_workbook(loc)
        except Exception as e:
            print(e)
            hmcCloseConnAndExitInstall(hmcObj)
        sheet = wb[wb.sheetnames[0]]
        maxCol = sheet.max_column
        maxRow = sheet.max_row
        keyInfo = []
        for i in range(1, maxRow+1):
            cellObj = sheet.cell(i, 1)
            if i > 1:
                keyInfo.append(cellObj.value)
        viosValidationList = []
        validated = False
        count = 0
        print("%-40s %-50s %-5s %-10s" % (
            "FIELDS NAME", "VALUE", "STATUS", "COMMENTS"))
        vios_count = 0
        for i in range(1, maxCol+1):
            viosInstallerInfo = {}
            viosHeaderName = ""
            for j in range(1, maxRow+1):
                if i == 1:
                    continue
                if j == 1:
                    continue
                viosHeaderName = sheet.cell(1, column=i).value
                cellObj = sheet.cell(row=j, column=i)
                viosInstallerInfo[keyInfo[j-2]] = cellObj.value
            if viosInstallerInfo:
                count += 1
                vios_count = vios_count + 1
                res = validateViosInstallerInfo(
                    hmcObj, viosInstallerInfo, viosHeaderName)
                if res == 0:
                    res = validateISOPath(
                        hmcObj, viosInstallerInfo['ISO PATH(HMC localpath)'])
                    if res == 0:
                        iolist = parseIOadapters(viosInstallerInfo)
                        selectedIO = validateIOinfo(
                            hmcObj, iolist, viosInstallerInfo, vios_count)
                        if selectedIO:
                            viosInstallerInfo['selected_io'] = selectedIO
                            validated = False
                            viosValidationList.append(viosInstallerInfo)
                            validated = True
        if len(viosValidationList) == count:
            if validated is False:
                hmcCloseConnAndExitInstall(hmcObj)
        else:
            hmcCloseConnAndExitInstall(hmcObj)

        print("\n:::: VALIDATING IO ADAPTERS DUPLICATE SELECTION ::::")
        isValidIOSelected = validateAdapterSelection(
            hmcObj, viosValidationList)
        if not isValidIOSelected:
            hmcCloseConnAndExitInstall(hmcObj)

        # Novalink xlsx Validation Start
        nlObj = validateNovalink.NovalinkInputValidator(hmcObj)
        nlSheetNo = 1
        res = nlObj.parseNovalinkXlsx(wb, nlSheetNo)
        if res[0] == 1:
            hmcCloseConnAndExitInstall(hmcObj)
        NLinfo = res[1]
        res = nlObj.validateNovalink(NLinfo)
        # Novalink xlsx Validation End
        if res > 0:
            hmcCloseConnAndExitInstall(hmcObj)

        print(":::: IP ADDRESS VALIDATING ::::")
        IP_ERROR = validateIPAddress(viosValidationList, NLinfo)
        if IP_ERROR:
            hmcCloseConnAndExitInstall(hmcObj)

        installvios = installVios.VIOSInstaller(hmcObj)
        isLparExist = checkExistingLPAR(viosValidationList, installvios)
        if isLparExist:
            hmcCloseConnAndExitInstall(hmcObj)

        # Release CO Management from Master
        frameName = viosValidationList[0]['HOST NAME']
        SHARED_PROC_POOL_ID = CONF.get('vios_profile', 'SHARED_PROC_POOL_ID')
        SHARED_PROC_POOL_NAME = CONF.get(
            'vios_profile', 'SHARED_PROC_POOL_NAME')

        try:
            res = hmcObj.coMgmtReleaseMaster(frameName)
            if res['code'] == 0:
                print(MSGCATALOG.REL_CO_MGMT_MASTER)
                time.sleep(15)
            else:
                for err in res['stdout']:
                    log.error(err)
        except Exception as e:
            log.error(e)

        # Create Shared Proc Pool for VIOS and Novalink
        try:
            res = hmcObj.changeSharedProcPoolAttr(
                frameName, SHARED_PROC_POOL_ID, SHARED_PROC_POOL_NAME)
            if res['code'] == 0:
                print(MSGCATALOG.SHARED_PROC_POOL_CONFIGURED)
            if res['code'] != 0:
                for err in res['stdout']:
                    print(err)
                    OPERATION_NOT_ALLOWED = CONF.get(
                        'hmc_error', 'OPERATION_NOT_ALLOWED')
                    if OPERATION_NOT_ALLOWED in err:
                        print(MSGCATALOG.HMC_NOT_OPERATING)
                        hmcCloseConnAndExitInstall(hmcObj)
        except Exception as e:
            log.error(e)

        # Create Management Switch for VIOS and Novalink on HMC
        try:
            MGMT_VSWITCH_NAME = CONF.get(
                'vios_profile', 'MGMT_VSWITCH_NAME')
            res = hmcObj.create_mgmt_vswitch(
                frameName, MGMT_VSWITCH_NAME)
            if res['code'] == 0:
                print(MSGCATALOG.MGMT_VSWITCH_CREATED)
            if res['code'] != 0:
                for err in res['stdout']:
                    print(err.split(".")[0])
        except Exception as e:
            log.error(e)

        # Create VIOS Profile and validate Network, Ping test
        viosInstallerList = []
        res = profAndNetworkVald(
            viosValidationList, installvios, viosInstallerList, HMC_IP)
        if not res:
            hmcCloseConnAndExitInstall(hmcObj)

        # Prepare data for Novalink Installation
        # Create Novalink Profile
        nlInstallerObj = installNovalink.NovalinkInstaller(hmcObj)
        nlInstallerInfo = nlInstallerObj.prepareNovalinkInstaller(
            NLinfo, viosInstallerList)
        res = nlInstallerObj.createNovalinkProfile(nlInstallerInfo)
        if not res:
            hmcCloseConnAndExitInstall(hmcObj)

        # Install VIOS trigger
        install = installVIOS(viosInstallerList, installvios)
        if not install:
            hmcCloseConnAndExitInstall(hmcObj)

        isRMCActive = checkRMC(viosInstallerList, installvios)
        if not isRMCActive:
            hmcCloseConnAndExitInstall(hmcObj)

        isRestarted = restartAndCreateSEA(viosInstallerList, installvios)
        if not isRestarted:
            hmcCloseConnAndExitInstall(hmcObj)

        isPrivateNwAdded = add_private_network(viosInstallerList, installvios)

        # Prepare VIOS for novalink installation
        # Create LV, map vHost to disk
        isVIOSready = installvios.prepareVIOSforNovalink(
            viosInstallerList, nlInstallerInfo)
        if not isVIOSready:
            hmcCloseConnAndExitInstall(hmcObj)

        # Set CO Management as Master
        # Set Novalink LPAR as PowerVM Management Capable
        res = nlInstallerObj.setCoMgmtAndPowerVMCapable(nlInstallerInfo)
        if not res:
            hmcCloseConnAndExitInstall(hmcObj)
        res = nlInstallerObj.installNovalink(nlInstallerInfo)
        if not res:
            hmcCloseConnAndExitInstall(hmcObj)

        print("Novalink install starting, Please wait ...")
        hmcObj.closeSSHconnection()
        SLEEP_TIME_BEFORE_NL_INSTALL_START = CONF.get(
            'novalink_profile', 'SLEEP_TIME_BEFORE_NL_INSTALL_START')
        time.sleep(int(SLEEP_TIME_BEFORE_NL_INSTALL_START))
        NL_IP = nlInstallerInfo['lparIp']
        NL_Hostname = nlInstallerInfo['hostname']
        print("Executing backend process to check SSH to NL")
        hmc_cred = {"ip": HMC_IP, "port": SSH_PORT, "uname": str(
            HMC_USERNAME), "pwd": HMC_PASSWORD}
        processP1 = Process(target=checkSshToNl, args=(hmcObj, NL_IP, str(
            NL_Hostname), nlInstallerObj, nlInstallerInfo, hmc_cred))
        processP1.start()
        print("Redirectig to NL Console")
        shPath = CONF.get('novalink_profile', 'NOVALINK_VTREM_SCRIPT_PATH')
        log.info("Proceeding for VTMENU of Novalink")
        subprocess.call([shPath, HMC_IP, HMC_USERNAME,
                        HMC_PASSWORD, nlInstallerInfo['frameName'],
                        nlInstallerInfo['lparName']])
