# =====================================================================
# (C) 2019-2021 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
from novalinkAutoInstaller.src.EndPoint import HMC_Handler
from openpyxl import Workbook
from openpyxl.worksheet.datavalidation import DataValidation
from openpyxl.utils import get_column_letter
from openpyxl.styles import PatternFill
from openpyxl.styles.borders import Border, Side, BORDER_THICK
import os
import sys
import json
import time
import six
from six.moves import configparser
CONF = configparser.ConfigParser()
CONF.read(autoLogger.configPath)
log = autoLogger.AutoLogger("PopulateTemplateModule")


cellBdrColor = Border(
        left=Side(border_style=BORDER_THICK, color='FFEE00'),
        right=Side(border_style=BORDER_THICK, color='FFEE00'),
        top=Side(border_style=BORDER_THICK, color='FFEE00'),
        bottom=Side(border_style=BORDER_THICK, color='FFEE00'),
        )
headerColor = PatternFill(
        start_color='FFEE00', end_color='FFEE00', fill_type='solid')


# Fetching LPAR list within a managed system.
# If any LPAR exist with a managed system, printing console log
# and exiting from installation.
def checkExistingLpar(hmcObj, frameName):
    log.info("Inside checkExistingLpar function")
    getLparList = hmcObj.listLpar(frameName)
    if len(getLparList) > 0:
        print("\n"+MSGCATALOG.LPAR_EXIST.format(frameName))
        log.info(MSGCATALOG.LPAR_EXIST)
        for info in getLparList:
            print("VM name : "+info['lpar_name']+', state : '+info['state'])
            log.info("VM name : "+info['lpar_name']+', state : '+info['state'])
        return True
    else:
        log.info("No VM found in Host, proceeding for xlsx generation")
        return False


def setSRIOVMode(hmcObj, ioName, frameName, sriov):
    log.info("Inside setSRIOVMode method")
    drcName = ioName.split('|')[0].strip()
    res = hmcObj.getDrcIndexUsingDrcName(frameName, drcName)
    drcIndex = res['drc_index']
    res = hmcObj.setSRIOVMode(frameName, drcIndex, sriov)
    if res['code'] == 0:
        if sriov == "sriov":
            print(MSGCATALOG.SRIOV_SHARED_MODE_SUCCESS.format(drcName))
            log.info("SRIOV Mode changed to Shared for Adapter: "+str(drcName))
        else:
            print(MSGCATALOG.SRIOV_DEDICATED_MODE_SUCCESS.format(drcName))
            log.info("SRIOV Mode changed to Dedicated for Adapter: "+str(
                drcName))

        return True
    else:
        for info in res['stdout']:
            if sriov == "sriov":
                SRIOV_ALREADY_SHARED = CONF.get(
                    'vios_profile', 'SRIOV_ALREADY_SHARED_MODE')
                if SRIOV_ALREADY_SHARED in info:
                    log.info(info)
                    msg = info.replace(drcIndex, drcName)
                    print(msg)
                    return True
                else:
                    print(MSGCATALOG.SRIOV_SHARED_MODE_FAILED.format(
                        drcName))
                    print(info)
                    log.error(info)
                    return False
            else:
                SRIOV_ALREADY_DEDICATED = CONF.get(
                    'vios_profile', 'SRIOV_ALREADY_DEDICATED_MODE')
                if SRIOV_ALREADY_DEDICATED in info:
                    log.info(info)
                    msg = info.replace(drcIndex, drcName)
                    print(msg)
                    return True
                else:
                    print(MSGCATALOG.SRIOV_DEDICATED_MODE_FAILED.format(
                        drcName))
                    print(info)
                    log.error(info)
                    return False


# Getting Physical IO Slot list
# Sorting IO SLOTS, keeping empty slots at end of list
def getIoAdapters(hmcObj, frameName, sriov):
    log.info("Inside getIoAdapters function")
    sas_adapter = CONF.get('vios_profile', 'IO_ADAPTERS_STRING_SAS').split(",")
    serial_bus = CONF.get('vios_profile', 'IO_ADAPTERS_STR_SERIAL')
    expansion_drawer = CONF.get('vios_profile', 'IO_ADAPTERS_EXPANSION_DRAWER')
    graphic_adapter = CONF.get('vios_profile', 'IO_GRAPHIC_ADAPTER')
    SRIOV_KEYWORD = CONF.get('vios_profile', 'SRIOV_KEYWORD')
    print(MSGCATALOG.HARDWARE_DETAILS_SCAN.format(frameName)+"\n")
    sriovRes = hmcObj.getSRIOVadapters(frameName)
    res = hmcObj.getIoSlots(frameName)
    io_slots = []
    io_slots_empty = []
    io_sriov = []
    for data in res:
        is_sas_adapter = False
        for info in sas_adapter:
            if info in data['description']:
                io_slots.insert(0, data['drc_name']+" | "+data['description'])
                is_sas_adapter = True
                break
        if is_sas_adapter:
            continue
        elif 'Empty' in data['description']:
            io_slots_empty.append(data['drc_name']+" | "+data['description'])
        elif serial_bus in data['description'] or expansion_drawer in str(
                data['description']) or graphic_adapter in data['description']:
            pass
        else:
            isSRIOV = False
            for sriovInfo in sriovRes:
                if str(sriovInfo.get('slot_id')).lower() == str(
                        data['drc_index']).lower():
                    ioName = data['drc_name']
                    print(MSGCATALOG.SRIOV_FOUND.format(ioName))
                    if sriov == "sriov":
                        print(MSGCATALOG.SRIOV_CONVERT_SRIOV_MODE)
                        SRIOVresponse = setSRIOVMode(
                            hmcObj, ioName, frameName, sriov)
                        if SRIOVresponse:
                            for x in range(15):
                                portList = hmcObj.get_sriov_adapter_ports(
                                    frameName, ioName)
                                breakForLoop = False
                                if len(portList) == 0:
                                    log.info("sriov port not available yet")
                                    log.info("trying again after 10 seconds")
                                    time.sleep(10)
                                    print("Please wait...")
                                    continue
                                log.info("SR-IOV shared port found")
                                log.info("All sriov ports list : "+str(
                                    portList))
                                for sr in portList:
                                    linkStatus = "Down"
                                    type = sr['port_type']
                                    if int(sr['state']) == 1:
                                        linkStatus = "Up"
                                    io_sriov.append(
                                        sr['phys_port_loc']+" | ("+str(
                                            SRIOV_KEYWORD)+"-"+str(
                                                type)+" Link - "+str(
                                                    linkStatus)+")")
                                    breakForLoop = True
                                if len(portList) == 0:
                                    print(MSGCATALOG.SRIOV_NOT_FOUND_WARNING)

                                if breakForLoop:
                                    break
                        isSRIOV = True
                    else:
                        print(MSGCATALOG.SRIOV_CONVERT_DEDICATED_MODE)
                        SRIOVresponse = setSRIOVMode(
                            hmcObj, ioName, frameName, sriov)
            if not isSRIOV:
                io_slots.append(data['drc_name']+" | "+data['description'])

    io_slots.extend(io_sriov)
    log.info("All IO Slots list : "+str(io_slots))
    return {"io_slots": io_slots}


def generateViosSheet(wb, xlsxHeaderVios, rawDataVios, res, noOfVios):
    log.info("Inside setViosParameters function")
    # creating xlsx and sheets and filling colors, font etc
    worksheetVios = wb.active
    worksheetVios = wb.create_sheet(
        CONF.get('install_template', 'NORMAL_MODE_VIOS_SHEET_NAME'), 0)
    worksheetVios.sheet_properties.tabColor = "1072BA"
    worksheetVios.cell(1, 1, 'PARAMETERS').fill = headerColor

    # Creating number of colums based on number of VIOS
    for x in range(noOfVios):
        worksheetVios.cell(1, x+2, 'VIOS'+str(x+1)).fill = headerColor

    # setting cell values in xlsx
    # Showing list of all IO Adapters for user selection
    rowCounter = 0
    setIoSlot = False
    XLSX_DIFFERENTIATOR = CONF.get('install_template', 'XLSX_DIFFERENTIATOR')
    for info in xlsxHeaderVios:
        # Setting 1st column of xlsx
        # worksheetVios.cell(rowCounter+2, 1, info)
        colList = ["ISO PATH(HMC localpath)", "IP ADDRESS",
                   "GATEWAY", "SUBNET MASK", "PORT VLAN ID",
                   XLSX_DIFFERENTIATOR]
        if info in colList:
            worksheetVios.cell(rowCounter+2, 1).border = cellBdrColor
        colKeys = '"'+info+'"'
        dataVal = DataValidation(
                        type='list', formula1=colKeys)
        worksheetVios.add_data_validation(dataVal)
        dataVal.add(get_column_letter(1)+str(rowCounter+2))
        worksheetVios.cell(rowCounter+2, 1, info)

        for x in range(noOfVios):
            if 'NAME' == info:
                worksheetVios.cell(
                    rowCounter+2, x+2, rawDataVios[rowCounter]+str(x+1))
            else:
                if XLSX_DIFFERENTIATOR == info:
                    setIoSlot = True
                    worksheetVios.cell(rowCounter+2, x+2, "")
                    continue
                if setIoSlot:
                    if 'Empty' in info:
                        dataVal = DataValidation(
                            type='list', formula1='"--"')
                    else:
                        dataVal = DataValidation(
                            type='list', formula1='"--,Required"')
                    worksheetVios.add_data_validation(dataVal)
                    dataVal.add(get_column_letter(x+2)+str(rowCounter+2))
                    worksheetVios.cell(rowCounter+2, x+2, " ")
                else:
                    if info in colList:
                        worksheetVios.cell(
                            rowCounter+2, x+2).border = cellBdrColor
                    else:
                        worksheetVios.cell(
                            rowCounter+2, x+2, rawDataVios[rowCounter])

        rowCounter = rowCounter + 1

    log.info("xlsx formatting for VIOS has been done")


def setViosParameters(hmcObj, wb, hmcIp, frameName, noOfVios, sriov):
    log.info("Inside setViosParameters function")
    VIOS_NAME = CONF.get('vios_profile', 'VIOS_NAME')
    VIOS_MIN_MEM = CONF.get('vios_profile', 'MIN_MEM')
    VIOS_DESIRED_MEM = CONF.get('vios_profile', 'DESIRED_MEM')
    VIOS_MAX_MEM = CONF.get('vios_profile', 'MAX_MEM')

    MIN_VIRTUAL_PROCS = CONF.get('vios_profile', 'MIN_VIRTUAL_PROCS')
    DESIRED_VIRTUAL_PROCS = CONF.get('vios_profile', 'DESIRED_VIRTUAL_PROCS')
    MAX_VIRTUAL_PROCS = CONF.get('vios_profile', 'MAX_VIRTUAL_PROCS')
    MIN_SHARED_PROC_UNITS = CONF.get('vios_profile', 'MIN_SHARED_PROC_UNITS')
    DESIRED_SHARED_PROC_UNITS = CONF.get(
        'vios_profile', 'DESIRED_SHARED_PROC_UNITS')
    MAX_SHARED_PROC_UNITS = CONF.get('vios_profile', 'MAX_SHARED_PROC_UNITS')

    # Creating list of all header values for VIOS
    xlsxHeaderVios = CONF.get(
        'install_template', 'FIELDS_NAME_VIOS').split(",")

    # Fethcing IO Adpaters list
    res = getIoAdapters(hmcObj, frameName, sriov)

    viosSharedProcessors = [MIN_VIRTUAL_PROCS, DESIRED_VIRTUAL_PROCS,
                            MAX_VIRTUAL_PROCS, MIN_SHARED_PROC_UNITS,
                            DESIRED_SHARED_PROC_UNITS, MAX_SHARED_PROC_UNITS]

    # Creating list of all default values from config file for VIOS
    rawDataVios = [VIOS_NAME, frameName, VIOS_MIN_MEM,
                   VIOS_DESIRED_MEM, VIOS_MAX_MEM]
    rawDataVios.extend(viosSharedProcessors)
    remainingRawDataVios = ['', '', '', '', '']
    rawDataVios.extend(remainingRawDataVios)
    xlsxHeaderVios.append(
        CONF.get('install_template', 'XLSX_DIFFERENTIATOR'))
    xlsxHeaderVios.extend(res['io_slots'])
    generateViosSheet(wb, xlsxHeaderVios, rawDataVios, res, noOfVios)


def generateNovalinkSheet(wb, xlsxHeaderLpar, rawDataLpar):
    log.info("Inside setNovalinkParameters function")
    # creating xlsx and sheets and filling colors, font etc
    worksheetNl = wb.create_sheet(
        CONF.get('install_template', 'NORMAL_MODE_NOVALINK_SHEET_NAME'), 1)
    worksheetNl.sheet_properties.tabColor = "1072BA"
    worksheetNl.cell(1, 1, 'PARAMETERS').fill = headerColor
    worksheetNl.cell(1, 2, 'VALUES').fill = headerColor

    # setting cell values for novalink sheet
    rowCounterLpar = 0
    colList = ["IP ADDRESS", "GATEWAY", "SUBNET MASK",
               "BOOTP SERVER IP", "PORT VLAN ID", "HOSTNAME(FQDN)"]
    for info in xlsxHeaderLpar:
        colKeys = '"'+info+'"'
        dataVal = DataValidation(
                        type='list', formula1=colKeys)
        worksheetNl.add_data_validation(dataVal)
        dataVal.add(get_column_letter(1)+str(rowCounterLpar+2))
        if info in colList:
            worksheetNl.cell(rowCounterLpar + 2, 1).border = cellBdrColor
        worksheetNl.cell(rowCounterLpar + 2, 1, info)
        if info in colList:
            worksheetNl.cell(rowCounterLpar + 2, 2).border = cellBdrColor
        else:
            worksheetNl.cell(
                rowCounterLpar + 2, 2, rawDataLpar[rowCounterLpar])
        rowCounterLpar = rowCounterLpar + 1


def setNovalinkParameters(wb, hmcIp, frameName):
    log.info("Inside generateNovalinkSheet function")
    LPAR_NAME = CONF.get('novalink_profile', 'LPAR_NAME')
    LPAR_MIN_MEM = CONF.get('novalink_profile', 'LPAR_MIN_MEM')
    LPAR_DESIRED_MEM = CONF.get('novalink_profile', 'LPAR_DESIRED_MEM')
    LPAR_MAX_MEM = CONF.get('novalink_profile', 'LPAR_MAX_MEM')

    LPAR_MIN_VIRTUAL_PROCS = CONF.get(
        'novalink_profile', 'LPAR_MIN_VIRTUAL_PROCS')
    LPAR_DESIRED_VIRTUAL_PROCS = CONF.get(
        'novalink_profile', 'LPAR_DESIRED_VIRTUAL_PROCS')
    LPAR_MAX_VIRTUAL_PROCS = CONF.get(
        'novalink_profile', 'LPAR_MAX_VIRTUAL_PROCS')
    LPAR_MIN_SHARED_PROC_UNITS = CONF.get(
        'novalink_profile', 'LPAR_MIN_SHARED_PROC_UNITS')
    LPAR_DESIRED_SHARED_PROC_UNITS = CONF.get(
        'novalink_profile', 'LPAR_DESIRED_SHARED_PROC_UNITS')
    LPAR_MAX_SHARED_PROC_UNITS = CONF.get(
        'novalink_profile', 'LPAR_MAX_SHARED_PROC_UNITS')

    LPAR_DISK_SIZE = CONF.get('novalink_profile', 'LPAR_DISK_SIZE')
    LPAR_DISK_NAME = CONF.get('novalink_profile', 'LPAR_DISK_NAME')
    LPAR_DISK_TO_USE = CONF.get('novalink_profile', 'LPAR_DISK_TO_USE')
    # Creating list of all header values for Novalink
    xlsxHeaderLpar = CONF.get(
        'install_template', 'FIELDS_NAME_LPAR').split(",")

    lparSharedProcessor = [LPAR_MIN_VIRTUAL_PROCS, LPAR_DESIRED_VIRTUAL_PROCS,
                           LPAR_MAX_VIRTUAL_PROCS, LPAR_MIN_SHARED_PROC_UNITS,
                           LPAR_DESIRED_SHARED_PROC_UNITS,
                           LPAR_MAX_SHARED_PROC_UNITS]
    rawDataLpar = [LPAR_NAME, LPAR_MIN_MEM, LPAR_DESIRED_MEM, LPAR_MAX_MEM]
    remainingRawDataLpar = [LPAR_DISK_SIZE, '', '', '', '', '', '', '',
                            '', '', '', '']

    rawDataLpar.extend(lparSharedProcessor)
    rawDataLpar.extend(remainingRawDataLpar)
    generateNovalinkSheet(wb, xlsxHeaderLpar, rawDataLpar)


def templateNormalMode(hmcObj, hmcIp, frameName, noOfVios, sriov):
    log.info("Inside templateNormalMode function")
    wb = Workbook()
    setViosParameters(hmcObj, wb, hmcIp, frameName, noOfVios, sriov)
    setNovalinkParameters(wb, hmcIp, frameName)
    log.info("xlsx formatting for Novalink has been done")
    # saving the xlsx file
    wb.save(CONF.get('install_template', 'TEMPLATE_FILE_NAME')+'.xlsx')
    return True


# Main function for generating xlsx template
# Reading all default values from config file
# Reading all hardware details from managed system
# Creating xlsx sheets for VIOS and Novalink
# Dynamic column creation based on number of VIOS to be created

def generateInstallerTemplate(hmcObj, hmcIp, frameName, mode, noOfVios, sriov):
    log.info("Inside generateInstallerTemplate function")
    if mode == 'normal':
        res = templateNormalMode(hmcObj, hmcIp, frameName, noOfVios, sriov)
        if res:
            log.info(MSGCATALOG.INSTALLER_TEMPLATE_GENERATED)
            xlsxPath = os.path.abspath(CONF.get(
                'install_template', 'TEMPLATE_FILE_NAME')+'.xlsx')
            print(MSGCATALOG.INSTALLER_TEMPLATE_GENERATED.format(str(
                xlsxPath))+"\n")
            closeConnection = hmcObj.closeSSHconnection()
            exit(0)
    else:
        print("Installation mode not supported!")
        log.error("Unable to generate Installer template")
        closeConnection = hmcObj.closeSSHconnection()
        exit(1)


# Main Function
def main(hmcIp, hmcUserName, hmcPassword, frameName, noOfVios, mode, sriov):
    log.info("Inside Populate template main function")
    hmcObj = HMC_Handler.HMChandler()
    SSH_PORT = CONF.get('ssh_config', 'SSH_PORT')
    connection = hmcObj.createSSHconnection(
        hmcIp, SSH_PORT, hmcUserName, hmcPassword)
    # printing console message in case of SSH Authentication error
    try:
        if connection.get("code") == 201:
            print(connection['error'])
            log.error("HMC SSH connection failed : "+str(connection['error']))
            exit(1)
    except Exception as e:
        log.info("HMC SSH connection successful")

    # Checking host for any exiting LPAR
    isLparExist = checkExistingLpar(hmcObj, frameName)
    if isLparExist:
        closeConnection = hmcObj.closeSSHconnection()
        exit(1)
    else:
        # Preparing xlsx
        generateInstallerTemplate(
            hmcObj, hmcIp, frameName, mode, noOfVios, sriov)
