# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
from novalinkAutoInstaller.src.Validate.validator import InputValidator
import os
import six
from six.moves import configparser
CONF = configparser.ConfigParser()
CONF.read(autoLogger.configPath)
log = autoLogger.AutoLogger("validateNovalinkModule")


# Comparing and validating xlsx template data from default
# configuration required for Novalink installation
# Validating hardware details from xlsx template
class NovalinkInputValidator(InputValidator):

    def __init__(self, hmcObj):
        self.hmcObj = hmcObj

    # Validate Numeric or decimal values for RAM, Processor
    def vldNumericVals(self, info):
        log.info("Inside vldNumericVals method")
        code = 0
        for infokey, infovalue in info.items():
            if infokey == "MIN MEMORY(GB)":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "DESIRED MEMORY(GB)":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MAX MEMORY(GB)":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MIN SHARED PROC UNITS":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "DESIRED SHARED PROC UNITS":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MAX SHARED PROC UNITS":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode

            # Value of Virtual processor should always
            # be a number, not float
            elif infokey == "MIN VIRTUAL PROCS":
                resCode = self.isDigitVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "DESIRED VIRTUAL PROCS":
                resCode = self.isDigitVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MAX VIRTUAL PROCS":
                resCode = self.isDigitVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "DISK SIZE(GB)":
                resCode = self.isDigitVals(infovalue, infokey)
                code = code + resCode

        return code

    # Mininum, desired and max memory validation
    # Desired should not be less than min memory
    # Max should not be less than desired memory
    def vldRAM(self, min, desired, max):
        log.info("Inside vldRAM method")
        VIOS_MIN_MEM = CONF.get('vios_profile', 'MIN_MEM')
        code = self.ramVld(VIOS_MIN_MEM, min, desired, max)
        return code

    # Mininum, desired and max Virtual Proc validation
    # Desired should not be less than min virtual proc
    # Max should not be less than desired virtual proc
    def vldVirtualProc(self, min, desired, max):
        log.info("Inside vldVirtualProc method")
        MIN_VIRTUAL_PROCS = CONF.get('vios_profile', 'MIN_VIRTUAL_PROCS')
        code = self.vProcVld(MIN_VIRTUAL_PROCS, min, desired, max)
        return code

    # Mininum, desired and max shared proc units validation
    # Desired should not be less than min shared proc units
    # Max should not be less than desired shared proc units
    def vldSharedProcUnits(self, min, desired, max):
        log.info("Inside vldSharedProcUnits method")
        MIN_SHARED_PROC_UNITS = CONF.get(
            'vios_profile', 'MIN_SHARED_PROC_UNITS')
        code = self.sharedProcVld(MIN_SHARED_PROC_UNITS, min, desired, max)
        return code

    # PORT VLAN ID validation
    # PORT VLAN ID should always be a +ve number between 0 to 4094
    def vldVLAN_ID(self, vlanId):
        log.info("Inside vldVLAN_ID method")
        maxRange = int(CONF.get('vios_profile', 'VLAN_ID_MAX_RANGE'))
        code = self.vlanVld(vlanId, maxRange,)
        return code

    # IP Address validation main function
    def vldNetworkAttributes(self, info):
        log.info("Inside vldNetworkAttributes method")
        viosIP = info['IP ADDRESS']
        resVIOSIP = self.ipAddrVld(viosIP)
        if resVIOSIP == 0:
            resVIOSIP = self.ipAddPingVld('IP ADDRESS', viosIP)
        else:
            self.valOutput('IP ADDRESS', 'F', viosIP, str(
                MSGCATALOG.IP_ADDRESS_INVALID))

        gatewayIp = info['GATEWAY']
        resGateway = self.ipAddrVld(gatewayIp)
        if resGateway == 0:
            resGateway = self.ipAddPingVld('GATEWAY', gatewayIp, True)
        else:
            self.valOutput('GATEWAY', 'F', gatewayIp, str(
                MSGCATALOG.IP_ADDRESS_INVALID))

        subnetMask = info['SUBNET MASK']
        resSMask = self.ipAddrVld(subnetMask)
        if resSMask == 0:
            self.valOutput('SUBNET MASK', 'P', subnetMask)
        else:
            self.valOutput('SUBNET MASK', 'F', subnetMask, str(
                MSGCATALOG.IP_ADDRESS_INVALID))

        bootP = info['BOOTP SERVER IP']
        resBootP = self.ipAddrVld(bootP)
        if resBootP == 0:
            resBootP = self.ipAddPingVld('BOOTP SERVER IP', bootP, True)
        else:
            self.valOutput('BOOTP SERVER IP', 'F', bootP, str(
                MSGCATALOG.IP_ADDRESS_INVALID))
        res = resVIOSIP + resGateway + resSMask + resBootP
        return res

    # Pasring Novalink xlsx sheet
    def parseNovalinkXlsx(self, wb, n):
        print("\n::::::::::::::::::::::::::::::::::::")
        print("VALIDATING : Novalink")
        print("::::::::::::::::::::::::::::::::::::")
        print("\n:::: REQUIRED FIELD VALIDATING ::::")
        NLsheet = wb[wb.sheetnames[1]]
        maxRow = NLsheet.max_row
        novalinkInfo = {}
        isValidated = 0
        for i in range(1, maxRow+1):
            if i > 1:
                keyName = NLsheet.cell(i, 1).value
                val = NLsheet.cell(i, 2).value
                if val is None or val == "" or val is False:
                    self.valOutput(keyName, 'F', '', 'Required')
                    isValidated = 1
                else:
                    self.valOutput(keyName, 'P', val)
                    novalinkInfo[keyName] = val
        return (isValidated, novalinkInfo)

    # Validate Novalink info main function
    # Modularized as a separate function for each type of validation
    def validateNovalink(self, info):
        log.info("Inside validateNovalink method")
        print("\n:::: VALIDATING RESOURCES ::::")
        code = self.vldNumericVals(info)
        if code > 0:
            return code
        codeRAM = self.vldRAM(
            info['MIN MEMORY(GB)'], info['DESIRED MEMORY(GB)'],
            info['MAX MEMORY(GB)'])
        codeProc = self.vldVirtualProc(
            info['MIN VIRTUAL PROCS'], info['DESIRED VIRTUAL PROCS'],
            info['MAX VIRTUAL PROCS'])
        codeProcUnit = self.vldSharedProcUnits(
            info['MIN SHARED PROC UNITS'], info['DESIRED SHARED PROC UNITS'],
            info['MAX SHARED PROC UNITS'])
        codeVLAN = self.vldVLAN_ID(info['PORT VLAN ID'])
        codeNW = self.vldNetworkAttributes(info)
        code = codeRAM + codeProc + codeProcUnit + codeVLAN + codeNW
        return code
